﻿using System;
using System.Drawing;

using Iterator.Events;
using Iterator.Armor;
using Iterator.Shooting;

using Iterator.FactionInfo;

namespace Iterator.Units
{
    public abstract class GameUnit
    {
        public event EventHandler<PositionChangedEventArgs> PositionChanged;

        public UnitSide Side { get; set; /* Can set, a unit can switch sides */ }
        public UnitBehaviour Behaviour { get; set; }

        public string Description { get; protected set; }
        public UnitType Type { get; protected set; }
        public IArmorBehaviour Armor { get; protected set; }
        public IShootingBehaviour ShootingBehaviour { get; protected set; }

        public Point Position { get; protected set; }

        protected FactionDetailsFactory FactionDetails { get; private set; }

        // Unit classes each take care of their own Display
        public abstract void Display();

        public GameUnit(FactionDetailsFactory factionFactory)
        {
            FactionDetails = factionFactory;

            Side = factionFactory.GetSide();
            Behaviour = factionFactory.GetBehaviour();
            Description = "Undefined Game Unit";
        }

        protected void OnPositionChanged()
        {
            if (PositionChanged != null)
            {
                PositionChanged(this, new PositionChangedEventArgs(Position));
            }
        }

        public void PerformShoot()
        {
            ShootingBehaviour.Shoot();
        }

        public void Move(Point newLocation)
        {
            Position = newLocation;
            OnPositionChanged();
        }

        public void Kill()
        {
            // Kill the unit.
            Side = UnitSide.None;
            Description += ", Dead";
        }
    }
}
